type MapFn<T, K> = (input: T) => K;

/**
 * Map each element of an array using a defined callback function.
 * @param items The array to map.
 * @param fn the function mapper.
 * @example
 *    R.map([1, 2, 3], x => x * 10) // => [10, 20, 30]
 * @data_first
 */
export function map<T, K>(items: T[], fn: (input: T) => K): K[];

/**
 * Map each element of an array using a defined callback function.
 * @param fn the function mapper
 * @example
 *    R.map(x => x * 10)([1, 2, 3]) // => [10, 20, 30]
 * @data_last
 */
export function map<T, K>(fn: (input: T) => K): (items: T[]) => K[];

export function map<T, K>(
  arg1: T[] | MapFn<T, K>,
  arg2?: T[] | MapFn<T, K>
): K[] | ((items: T[]) => K[]) {
  if (arguments.length === 1) {
    return (items: T[]) => _map(items, arg1 as MapFn<T, K>);
  }
  return _map(arg1 as T[], arg2 as MapFn<T, K>);
}

function _map<T, K>(items: T[], fn: (input: T) => K) {
  return items.map(item => fn(item));
}
